package com.webflexexample.webflexExample.handler;

import com.webflexexample.webflexExample.Exception.CustomException;
import com.webflexexample.webflexExample.mapper.EmployeeDto;
import com.webflexexample.webflexExample.repository.EmployeeRepository;
import com.webflexexample.webflexExample.utils.EmployeeUtills;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.reactive.function.server.ServerResponse;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class EmployeeHandler {


    @Autowired
    private EmployeeRepository employeeRepository;
    public Mono<ServerResponse> getEmployee(ServerRequest serverRequest){
      String empId= serverRequest.pathVariable("id");
      Mono<EmployeeDto> employeeDtoMono=employeeRepository.findById(empId).map(EmployeeUtills::entityToDto).switchIfEmpty(Mono.error(new CustomException("object is empty")));
      return ServerResponse.ok().contentType(MediaType.TEXT_EVENT_STREAM).body(employeeDtoMono,EmployeeDto.class);
    }

    public Mono<ServerResponse> getAllEmployees(){
        Flux<EmployeeDto> getEmployees=employeeRepository.findAll().map(EmployeeUtills::entityToDto);
        return ServerResponse.ok().body(getEmployees,EmployeeDto.class);
    }

    public Mono<ServerResponse> saveEmployee(ServerRequest request) {
        Mono<EmployeeDto> employeeDtoMono = request.bodyToMono(EmployeeDto.class);
        Mono<EmployeeDto> saveToEntity = employeeDtoMono.map(EmployeeUtills::mapEmployeeDtoToEmployee).flatMap(employeeRepository::save).map(EmployeeUtills::entityToDto);
        return ServerResponse.ok().contentType(MediaType.TEXT_EVENT_STREAM).body(saveToEntity, EmployeeDto.class);
    }
}
